/****************************************************************************************************************************
  WebClientRepeating.ino

  For Portenta_H7 (STM32H7) with Vision-Shield Ethernet or Murata WiFi

  Portenta_H7_AsyncWebServer is a library for the Portenta_H7 with Vision-Shield Ethernet or Murata WiFi

  Based on and modified from ESPAsyncWebServer (https://github.com/me-no-dev/ESPAsyncWebServer)
  Built by Khoi Hoang https://github.com/khoih-prog/Portenta_H7_AsyncWebServer
  Licensed under GPLv3 license
 *****************************************************************************************************************************/

#include "defines.h"

char server[] = "arduino.cc";

unsigned long lastConnectionTime = 0;         // last time you connected to the server, in milliseconds
const unsigned long postingInterval = 10000L; // delay between updates, in milliseconds

// Initialize the Web client object
WiFiClient client;

// this method makes a HTTP connection to the server
void httpRequest()
{
	Serial.println();

	// close any connection before send a new request
	// this will free the socket on the WiFi shield
	client.stop();

	// if there's a successful connection
	if (client.connect(server, 80))
	{
		Serial.println(F("Connecting..."));

		// send the HTTP PUT request
		client.println(F("GET /asciilogo.txt HTTP/1.1"));
		client.println(F("Host: arduino.cc"));
		client.println(F("Connection: close"));
		client.println();

		// note the time that the connection was made
		lastConnectionTime = millis();
	}
	else
	{
		// if you couldn't make a connection
		Serial.println(F("Connection failed"));
	}
}

void printWifiStatus()
{
	// print the SSID of the network you're attached to:
	Serial.print("SSID: ");
	Serial.println(WiFi.SSID());

	// print your board's IP address:
	IPAddress ip = WiFi.localIP();
	Serial.print("Local IP Address: ");
	Serial.println(ip);

	// print the received signal strength:
	long rssi = WiFi.RSSI();
	Serial.print("signal strength (RSSI):");
	Serial.print(rssi);
	Serial.println(" dBm");
}

void setup()
{
	// Open serial communications and wait for port to open:
	Serial.begin(115200);

	while (!Serial && millis() < 5000);

	Serial.print("\nStart WebClientRepeating on ");
	Serial.print(BOARD_NAME);
	Serial.print(" with ");
	Serial.println(SHIELD_TYPE);
	Serial.println(PORTENTA_H7_ASYNC_TCP_VERSION);
	Serial.println(PORTENTA_H7_ASYNC_WEBSERVER_VERSION);

	///////////////////////////////////

	// check for the WiFi module:
	if (WiFi.status() == WL_NO_MODULE)
	{
		Serial.println("Communication with WiFi module failed!");

		// don't continue
		while (true);
	}

	Serial.print(F("Connecting to SSID: "));
	Serial.println(ssid);

	status = WiFi.begin(ssid, pass);

	delay(1000);

	// attempt to connect to WiFi network
	while ( status != WL_CONNECTED)
	{
		delay(500);

		// Connect to WPA/WPA2 network
		status = WiFi.status();
	}

	printWifiStatus();

	///////////////////////////////////
}

void loop()
{
	// if there's incoming data from the net connection send it out the serial port
	// this is for debugging purposes only
	while (client.available())
	{
		char c = client.read();
		Serial.write(c);
	}

	// if 10 seconds have passed since your last connection,
	// then connect again and send data
	if (millis() - lastConnectionTime > postingInterval)
	{
		httpRequest();
	}
}
